@@ -29,8 +29,6 @@
 
 #include "config.h"
 
-#include <wordexp.h>
-
 #include <vector>
 
 #include "json_ptr.hh"
@@ -337,6 +335,12 @@ string execute_sql(exec_context &ec, const string &sql, string &alt_msg)
         else if (sqlite3_stmt_readonly(stmt.in())) {
             retval = "No rows matched";
             alt_msg = "";
+
+            if (lnav_data.ld_flags & LNF_HEADLESS) {
+                if (ec.ec_local_vars.size() == 1) {
+                    ensure_view(&lnav_data.ld_views[LNV_DB]);
+                }
+            }
         }
 #endif
     }
@@ -527,6 +531,13 @@ string execute_from_file(exec_context &ec, const string &path, int line_number,
             retval = execute_command(ec, cmdline);
             break;
         case '/':
+            if (!lnav_data.ld_view_stack.empty()) {
+                lnav_view_t index = lnav_view_t(lnav_data.ld_view_stack.back() - lnav_data.ld_views);
+
+                execute_search(index, cmdline.substr(1));
+                retval = "";
+            }
+            break;
         case ';':
             setup_logline_table();
             retval = execute_sql(ec, cmdline, alt_msg);
@@ -560,6 +571,13 @@ string execute_any(exec_context &ec, const string &cmdline_with_mode)
             retval = execute_command(ec, cmdline);
             break;
         case '/':
+            if (!lnav_data.ld_view_stack.empty()) {
+                lnav_view_t index = lnav_view_t(lnav_data.ld_view_stack.back() - lnav_data.ld_views);
+
+                execute_search(index, cmdline.substr(1));
+                retval = "";
+            }
+            break;
         case ';':
             setup_logline_table();
             retval = execute_sql(ec, cmdline, alt_msg);
@@ -599,6 +617,12 @@ void execute_init_commands(exec_context &ec, vector<pair<string, string> > &msgs
             msg = execute_command(ec, cmd.substr(1));
             break;
         case '/':
+            if (!lnav_data.ld_view_stack.empty()) {
+                lnav_view_t index = lnav_view_t(lnav_data.ld_view_stack.back() - lnav_data.ld_views);
+
+                execute_search(index, cmd.substr(1));
+            }
+            break;
         case ';':
             setup_logline_table();
             msg = execute_sql(ec, cmd.substr(1), alt_msg);
@@ -679,7 +703,7 @@ int sql_callback(exec_context &ec, sqlite3_stmt *stmt)
         dls.push_column(value);
         if (value != NULL &&
             (dls.dls_headers[lpc].hm_name == "log_line" ||
-             dls.dls_headers[lpc].hm_name == "min(log_line)")) {
+             strstr(dls.dls_headers[lpc].hm_name.c_str(), "log_line"))) {
             int line_number = -1;
 
             if (sscanf(value, "%d", &line_number) == 1) {
