@@ -178,6 +178,10 @@ void handle_paging_key(int ch)
 
     /* process the command keystroke */
     switch (ch) {
+        case 0x7f:
+        case KEY_BACKSPACE:
+            break;
+
         case 'a':
             if (lnav_data.ld_last_view == NULL) {
                 alerter::singleton().chime();
@@ -1187,7 +1191,6 @@ void handle_paging_key(int ch)
 
         case KEY_CTRL_R:
             reset_session();
-            rebuild_indexes(true);
             break;
 
         case KEY_CTRL_W:
