@@ -36,7 +36,7 @@
 #include <stdio.h>
 #include <fcntl.h>
 #include <ctype.h>
-#include <wordexp.h>
+#include <stdarg.h>
 
 #include <fstream>
 
@@ -172,9 +172,13 @@ std::string time_ago(time_t last_time, bool convert_local)
         fmt    = "%d days ago";
         amount = delta / (24 * 60 * 60);
     }
-    else {
+    else if (delta < (2 * 365 * 24 * 60 * 60)) {
         return "over a year ago";
     }
+    else {
+        fmt = "over %d years ago";
+        amount = delta / (365 * 24 * 60 * 60);
+    }
 
     snprintf(buffer, sizeof(buffer), fmt, amount);
 
@@ -377,6 +381,26 @@ static const int year_lengths[2] = {
         366
 } ;
 
+static void secs2wday(const struct timeval &tv, struct tm *res)
+{
+    long days, rem;
+    time_t lcltime;
+
+    /* base decision about std/dst time on current time */
+    lcltime = tv.tv_sec;
+
+    days = ((long) lcltime) / SECSPERDAY;
+    rem = ((long) lcltime) % SECSPERDAY;
+    while (rem < 0) {
+        rem += SECSPERDAY;
+        --days;
+    }
+
+    /* compute day of week */
+    if ((res->tm_wday = ((EPOCH_WDAY + days) % DAYSPERWEEK)) < 0)
+        res->tm_wday += DAYSPERWEEK;
+}
+
 struct tm *secs2tm(time_t *tim_p, struct tm *res)
 {
     long days, rem;
@@ -461,34 +485,6 @@ bool next_format(const char * const fmt[], int &index, int &locked_index)
     return retval;
 }
 
-static const char *time_fmt_with_zone = "%a %b %d %H:%M:%S ";
-
-const char *std_time_fmt[] = {
-    "%Y-%m-%d %H:%M:%S",
-    "%Y-%m-%d %H:%M",
-    "%Y-%m-%dT%H:%M:%S",
-    "%Y-%m-%dT%H:%M:%SZ",
-    "%Y/%m/%d %H:%M:%S",
-    "%Y/%m/%d %H:%M",
-
-    "%a %b %d %H:%M:%S %Y",
-    "%a %b %d %H:%M:%S %Z %Y",
-    time_fmt_with_zone,
-
-    "%d/%b/%Y:%H:%M:%S +0000",
-    "%d/%b/%Y:%H:%M:%S %z",
-
-    "%b %d %H:%M:%S",
-
-    "%m/%d/%y %H:%M:%S",
-
-    "%m%d %H:%M:%S",
-
-    "+%s",
-
-    NULL,
-};
-
 const char *date_time_scanner::scan(const char *time_dest,
                                     size_t time_len,
                                     const char * const time_fmt[],
@@ -509,7 +505,9 @@ const char *date_time_scanner::scan(const char *time_dest,
                        this->dts_fmt_lock)) {
         *tm_out = this->dts_base_tm;
         tm_out->et_flags = 0;
-        if (time_dest[0] == '+') {
+        if (time_len > 1 &&
+            time_dest[0] == '+' &&
+            isdigit(time_dest[1])) {
             char time_cp[time_len + 1];
             int gmt_int, off;
 
@@ -547,8 +545,7 @@ const char *date_time_scanner::scan(const char *time_dest,
                 tm_out->et_tm.tm_zone = NULL;
             }
 #endif
-            if (func(tm_out, time_dest, off, time_len) &&
-                (time_dest[off] == '.' || time_dest[off] == ',' || off == time_len)) {
+            if (func(tm_out, time_dest, off, time_len)) {
                 retval = &time_dest[off];
 
                 if (tm_out->et_tm.tm_year < 70) {
@@ -562,6 +559,7 @@ const char *date_time_scanner::scan(const char *time_dest,
                 }
                 tv_out.tv_sec = tm2sec(&tm_out->et_tm);
                 tv_out.tv_usec = tm_out->et_nsec / 1000;
+                secs2wday(tv_out, &tm_out->et_tm);
 
                 this->dts_fmt_lock = curr_time_fmt;
                 this->dts_fmt_len  = retval - time_dest;
@@ -597,6 +595,7 @@ const char *date_time_scanner::scan(const char *time_dest,
 
                 tv_out.tv_sec = tm2sec(&tm_out->et_tm);
                 tv_out.tv_usec = tm_out->et_nsec / 1000;
+                secs2wday(tv_out, &tm_out->et_tm);
 
                 this->dts_fmt_lock = curr_time_fmt;
                 this->dts_fmt_len  = retval - time_dest;
@@ -694,36 +693,6 @@ bool read_file(const char *filename, string &out)
     return false;
 }
 
-bool wordexperr(int rc, string &msg)
-{
-    switch (rc) {
-        case WRDE_BADCHAR:
-            msg = "error: invalid filename character";
-            return false;
-
-        case WRDE_CMDSUB:
-            msg = "error: command substitution is not allowed";
-            return false;
-
-        case WRDE_BADVAL:
-            msg = "error: unknown environment variable in file name";
-            return false;
-
-        case WRDE_NOSPACE:
-            msg = "error: out of memory";
-            return false;
-
-        case WRDE_SYNTAX:
-            msg = "error: invalid syntax";
-            return false;
-
-        default:
-            break;
-    }
-
-    return true;
-}
-
 size_t abbreviate_str(char *str, size_t len, size_t max_len)
 {
     size_t last_start = 1;
