@@ -167,8 +167,11 @@ static int read_format_bool(yajlpp_parse_context *ypc, int val)
 
     if (field_name == "convert-to-local-time")
         elf->lf_date_time.dts_local_time = val;
-    else if (field_name == "json")
-        elf->jlf_json = val;
+    else if (field_name == "json") {
+        if (val) {
+            elf->elf_type = external_log_format::ELF_TYPE_JSON;
+        }
+    }
     else if (field_name == "hide-extra")
         elf->jlf_hide_extra = val;
     else if (field_name == "multiline")
@@ -426,6 +429,7 @@ static const json_path_handler_base::enum_value_t KIND_ENUM[] = {
     {"float", logline_value::VALUE_FLOAT},
     {"boolean", logline_value::VALUE_BOOLEAN},
     {"json", logline_value::VALUE_JSON},
+    {"struct", logline_value::VALUE_STRUCT},
     {"quoted", logline_value::VALUE_QUOTED},
 
     json_path_handler_base::ENUM_TERMINATOR
@@ -458,6 +462,7 @@ static struct json_path_handler value_def_handlers[] = {
         .for_field(&nullobj<external_log_format::value_def>()->vd_collate),
 
     json_path_handler("unit/")
+        .with_description("Unit definitions for this field")
         .with_children(unit_handlers),
 
     json_path_handler("identifier")
@@ -493,6 +498,35 @@ static struct json_path_handler value_def_handlers[] = {
     json_path_handler()
 };
 
+static struct json_path_handler highlighter_def_handlers[] = {
+    json_path_handler("pattern")
+        .with_synopsis("<regex>")
+        .with_description("A regular expression to highlight in logs of this format.")
+        .for_field(&nullobj<external_log_format::highlighter_def>()->hd_pattern),
+
+    json_path_handler("color")
+        .with_synopsis("#<hex>|<name>")
+        .with_description("The color to use when highlighting this pattern.")
+        .for_field(&nullobj<external_log_format::highlighter_def>()->hd_color),
+
+    json_path_handler("background-color")
+        .with_synopsis("#<hex>|<name>")
+        .with_description("The background color to use when highlighting this pattern.")
+        .for_field(&nullobj<external_log_format::highlighter_def>()->hd_background_color),
+
+    json_path_handler("underline")
+        .with_synopsis("<enabled>")
+        .with_description("Highlight this pattern with an underline.")
+        .for_field(&nullobj<external_log_format::highlighter_def>()->hd_underline),
+
+    json_path_handler("blink")
+        .with_synopsis("<enabled>")
+        .with_description("Highlight this pattern by blinking.")
+        .for_field(&nullobj<external_log_format::highlighter_def>()->hd_blink),
+
+    json_path_handler()
+};
+
 static const json_path_handler_base::enum_value_t LEVEL_ENUM[] = {
     make_pair(logline::level_names[logline::LEVEL_TRACE], logline::LEVEL_TRACE),
     make_pair(logline::level_names[logline::LEVEL_DEBUG5], logline::LEVEL_DEBUG5),
@@ -524,6 +558,14 @@ struct json_path_handler sample_handlers[] = {
     json_path_handler()
 };
 
+static const json_path_handler_base::enum_value_t TYPE_ENUM[] = {
+    make_pair("text", external_log_format::elf_type_t::ELF_TYPE_TEXT),
+    make_pair("json", external_log_format::elf_type_t::ELF_TYPE_JSON),
+    make_pair("csv", external_log_format::elf_type_t::ELF_TYPE_CSV),
+
+    json_path_handler_base::ENUM_TERMINATOR
+};
+
 struct json_path_handler format_handlers[] = {
     json_path_handler("regex/(?<pattern_name>[^/]+)/")
         .with_obj_provider(pattern_provider)
@@ -540,6 +582,10 @@ struct json_path_handler format_handlers[] = {
                               "body-field|url|url#|title|description|"
                               "timestamp-format#|module-field|opid-field)$",
                       read_format_field),
+    json_path_handler("ordered-by-time")
+        .with_synopsis("<bool>")
+        .with_description("Indicates that the order of messages in the file is time-based.")
+        .for_field(&nullobj<external_log_format>()->lf_time_ordered),
     json_path_handler("level/"
                       "(trace|debug\\d*|info|stats|warning|error|critical|fatal)")
         .add_cb(read_levels)
@@ -568,16 +614,23 @@ struct json_path_handler format_handlers[] = {
         .with_synopsis("<regex>")
         .with_description("The regular expression for this search table."),
 
-    json_path_handler("highlights#")
-        .with_synopsis("<regex>")
-        .with_description("A regular expression to highlight in logs of this format.")
-        .for_field(&nullobj<external_log_format>()->elf_highlighter_patterns),
+    json_path_handler("highlights/(?<highlight_name>[^/]+)/")
+        .with_obj_provider<external_log_format::highlighter_def, external_log_format>([](const yajlpp_provider_context &ypc, external_log_format *root) {
+            return &(root->elf_highlighter_patterns[ypc.get_substr_i(0)]);
+        })
+        .with_children(highlighter_def_handlers),
+
+    json_path_handler("file-type")
+        .with_synopsis("The type of file that contains the log messages")
+        .with_enum_values(TYPE_ENUM)
+        .for_enum(&nullobj<external_log_format>()->elf_type),
 
     json_path_handler()
 };
 
 struct json_path_handler root_format_handler[] = {
     json_path_handler("/(?<format_name>\\w+)/")
+        .with_description("The definition of a log file format.")
         .with_obj_provider(ensure_format)
         .with_children(format_handlers),
 
@@ -663,8 +716,9 @@ std::vector<intern_string_t> load_format_file(const string &filename, std::vecto
         char errmsg[1024];
 
         snprintf(errmsg, sizeof(errmsg),
-                "error: unable to open format file -- %s",
-                filename.c_str());
+                 "error:unable to open format file '%s' -- %s",
+                 filename.c_str(),
+                 strerror(errno));
         errors.push_back(errmsg);
     }
     else {
@@ -683,9 +737,11 @@ std::vector<intern_string_t> load_format_file(const string &filename, std::vecto
                 break;
             }
             else if (rc == -1) {
-                errors.push_back(filename +
-                        ":unable to read file -- " +
-                        string(strerror(errno)));
+                errors.push_back(
+                    "error:" +
+                    filename +
+                    ":unable to read file -- " +
+                    string(strerror(errno)));
                 break;
             }
             if (offset == 0 && (rc > 2) &&
@@ -694,18 +750,26 @@ std::vector<intern_string_t> load_format_file(const string &filename, std::vecto
                 buffer[0] = buffer[1] = '/';
             }
             if (ypc.parse((const unsigned char *)buffer, rc) != yajl_status_ok) {
-                errors.push_back(filename +
-                        ": invalid json -- " +
-                        string((char *)yajl_get_error(handle, 1, (unsigned char *)buffer, rc)));
+                errors.push_back(
+                    "error:" +
+                    filename +
+                    ":" +
+                    to_string(ypc.get_line_number()) +
+                    ":invalid json -- " +
+                    string((char *)yajl_get_error(handle, 1, (unsigned char *)buffer, rc)));
                 break;
             }
             offset += rc;
         }
         if (rc == 0) {
             if (ypc.complete_parse() != yajl_status_ok) {
-                errors.push_back(filename +
-                        ": invalid json -- " +
-                        string((char *)yajl_get_error(handle, 0, NULL, 0)));
+                errors.push_back(
+                    "error:" +
+                    filename +
+                    ":" +
+                    to_string(ypc.get_line_number()) +
+                    ":invalid json -- " +
+                    string((char *)yajl_get_error(handle, 0, NULL, 0)));
             }
         }
     }
@@ -896,7 +960,11 @@ static void exec_sql_in_path(sqlite3 *db, const string &path, std::vector<string
                 sql_execute_script(db, filename.c_str(), content.c_str(), errors);
             }
             else {
-                errors.push_back("Unable to read file: " + filename);
+                errors.push_back(
+                    "error:unable to read file '" +
+                    filename +
+                    "' -- " +
+                    string(strerror(errno)));
             }
         }
     }
