@@ -29,8 +29,6 @@
 
 #include "config.h"
 
-#include <wordexp.h>
-
 #include "lnav.hh"
 #include "lnav_util.hh"
 #include "sysclip.hh"
@@ -54,7 +52,6 @@ void rl_change(void *dummy, readline_curses *rc)
 
     tc->get_highlights().erase("$preview");
     tc->get_highlights().erase("$bodypreview");
-    tc->reload_data();
     lnav_data.ld_preview_source.clear();
     lnav_data.ld_preview_status_source.get_description().clear();
 
@@ -69,8 +66,7 @@ void rl_change(void *dummy, readline_curses *rc)
             if (!args.empty()) {
                 iter = lnav_commands.find(args[0]);
             }
-            if (iter == lnav_commands.end() ||
-                iter->second.c_description == NULL) {
+            if (iter == lnav_commands.end()) {
                 lnav_data.ld_doc_source.clear();
                 lnav_data.ld_example_source.clear();
                 lnav_data.ld_preview_source.clear();
