@@ -230,6 +230,57 @@ bool ptime_f15(struct exttm *dst, const char *str, off_t &off, ssize_t len) {
 }
 
 bool ptime_f16(struct exttm *dst, const char *str, off_t &off, ssize_t len) {
+    if (!ptime_Y(dst, str, off, len)) return false;
+    if (!ptime_char(' ', str, off, len)) return false;
+    if (!ptime_b(dst, str, off, len)) return false;
+    if (!ptime_char(' ', str, off, len)) return false;
+    if (!ptime_d(dst, str, off, len)) return false;
+    if (!ptime_char(' ', str, off, len)) return false;
+    if (!ptime_upto(' ', str, off, len)) return false;
+    if (!ptime_char(' ', str, off, len)) return false;
+    if (!ptime_H(dst, str, off, len)) return false;
+    if (!ptime_char(':', str, off, len)) return false;
+    if (!ptime_M(dst, str, off, len)) return false;
+    if (!ptime_char(':', str, off, len)) return false;
+    if (!ptime_S(dst, str, off, len)) return false;
+    if (!ptime_char('.', str, off, len)) return false;
+    if (!ptime_L(dst, str, off, len)) return false;
+    return true;
+}
+
+bool ptime_f17(struct exttm *dst, const char *str, off_t &off, ssize_t len) {
+    if (!ptime_Y(dst, str, off, len)) return false;
+    if (!ptime_char(' ', str, off, len)) return false;
+    if (!ptime_b(dst, str, off, len)) return false;
+    if (!ptime_char(' ', str, off, len)) return false;
+    if (!ptime_d(dst, str, off, len)) return false;
+    if (!ptime_char(' ', str, off, len)) return false;
+    if (!ptime_H(dst, str, off, len)) return false;
+    if (!ptime_char(':', str, off, len)) return false;
+    if (!ptime_M(dst, str, off, len)) return false;
+    if (!ptime_char(':', str, off, len)) return false;
+    if (!ptime_S(dst, str, off, len)) return false;
+    if (!ptime_char('.', str, off, len)) return false;
+    if (!ptime_L(dst, str, off, len)) return false;
+    return true;
+}
+
+bool ptime_f18(struct exttm *dst, const char *str, off_t &off, ssize_t len) {
+    if (!ptime_Y(dst, str, off, len)) return false;
+    if (!ptime_char(' ', str, off, len)) return false;
+    if (!ptime_b(dst, str, off, len)) return false;
+    if (!ptime_char(' ', str, off, len)) return false;
+    if (!ptime_d(dst, str, off, len)) return false;
+    if (!ptime_char(' ', str, off, len)) return false;
+    if (!ptime_H(dst, str, off, len)) return false;
+    if (!ptime_char(':', str, off, len)) return false;
+    if (!ptime_M(dst, str, off, len)) return false;
+    if (!ptime_char(':', str, off, len)) return false;
+    if (!ptime_S(dst, str, off, len)) return false;
+    return true;
+}
+
+bool ptime_f19(struct exttm *dst, const char *str, off_t &off, ssize_t len) {
     if (!ptime_upto(' ', str, off, len)) return false;
     if (!ptime_char(' ', str, off, len)) return false;
     if (!ptime_b(dst, str, off, len)) return false;
@@ -246,7 +297,7 @@ bool ptime_f16(struct exttm *dst, const char *str, off_t &off, ssize_t len) {
     return true;
 }
 
-bool ptime_f17(struct exttm *dst, const char *str, off_t &off, ssize_t len) {
+bool ptime_f20(struct exttm *dst, const char *str, off_t &off, ssize_t len) {
     if (!ptime_upto(' ', str, off, len)) return false;
     if (!ptime_char(' ', str, off, len)) return false;
     if (!ptime_b(dst, str, off, len)) return false;
@@ -265,7 +316,7 @@ bool ptime_f17(struct exttm *dst, const char *str, off_t &off, ssize_t len) {
     return true;
 }
 
-bool ptime_f18(struct exttm *dst, const char *str, off_t &off, ssize_t len) {
+bool ptime_f21(struct exttm *dst, const char *str, off_t &off, ssize_t len) {
     if (!ptime_upto(' ', str, off, len)) return false;
     if (!ptime_char(' ', str, off, len)) return false;
     if (!ptime_b(dst, str, off, len)) return false;
@@ -284,7 +335,7 @@ bool ptime_f18(struct exttm *dst, const char *str, off_t &off, ssize_t len) {
     return true;
 }
 
-bool ptime_f19(struct exttm *dst, const char *str, off_t &off, ssize_t len) {
+bool ptime_f22(struct exttm *dst, const char *str, off_t &off, ssize_t len) {
     if (!ptime_upto(' ', str, off, len)) return false;
     if (!ptime_char(' ', str, off, len)) return false;
     if (!ptime_b(dst, str, off, len)) return false;
@@ -300,7 +351,7 @@ bool ptime_f19(struct exttm *dst, const char *str, off_t &off, ssize_t len) {
     return true;
 }
 
-bool ptime_f20(struct exttm *dst, const char *str, off_t &off, ssize_t len) {
+bool ptime_f23(struct exttm *dst, const char *str, off_t &off, ssize_t len) {
     if (!ptime_upto(' ', str, off, len)) return false;
     if (!ptime_char(' ', str, off, len)) return false;
     if (!ptime_b(dst, str, off, len)) return false;
@@ -318,7 +369,7 @@ bool ptime_f20(struct exttm *dst, const char *str, off_t &off, ssize_t len) {
     return true;
 }
 
-bool ptime_f21(struct exttm *dst, const char *str, off_t &off, ssize_t len) {
+bool ptime_f24(struct exttm *dst, const char *str, off_t &off, ssize_t len) {
     if (!ptime_d(dst, str, off, len)) return false;
     if (!ptime_char('/', str, off, len)) return false;
     if (!ptime_b(dst, str, off, len)) return false;
@@ -339,7 +390,7 @@ bool ptime_f21(struct exttm *dst, const char *str, off_t &off, ssize_t len) {
     return true;
 }
 
-bool ptime_f22(struct exttm *dst, const char *str, off_t &off, ssize_t len) {
+bool ptime_f25(struct exttm *dst, const char *str, off_t &off, ssize_t len) {
     if (!ptime_d(dst, str, off, len)) return false;
     if (!ptime_char('/', str, off, len)) return false;
     if (!ptime_b(dst, str, off, len)) return false;
@@ -356,7 +407,90 @@ bool ptime_f22(struct exttm *dst, const char *str, off_t &off, ssize_t len) {
     return true;
 }
 
-bool ptime_f23(struct exttm *dst, const char *str, off_t &off, ssize_t len) {
+bool ptime_f26(struct exttm *dst, const char *str, off_t &off, ssize_t len) {
+    if (!ptime_d(dst, str, off, len)) return false;
+    if (!ptime_char('-', str, off, len)) return false;
+    if (!ptime_b(dst, str, off, len)) return false;
+    if (!ptime_char('-', str, off, len)) return false;
+    if (!ptime_Y(dst, str, off, len)) return false;
+    if (!ptime_char(' ', str, off, len)) return false;
+    if (!ptime_H(dst, str, off, len)) return false;
+    if (!ptime_char(':', str, off, len)) return false;
+    if (!ptime_M(dst, str, off, len)) return false;
+    if (!ptime_char(':', str, off, len)) return false;
+    if (!ptime_S(dst, str, off, len)) return false;
+    if (!ptime_char(' ', str, off, len)) return false;
+    if (!ptime_z(dst, str, off, len)) return false;
+    return true;
+}
+
+bool ptime_f27(struct exttm *dst, const char *str, off_t &off, ssize_t len) {
+    if (!ptime_d(dst, str, off, len)) return false;
+    if (!ptime_char('-', str, off, len)) return false;
+    if (!ptime_b(dst, str, off, len)) return false;
+    if (!ptime_char('-', str, off, len)) return false;
+    if (!ptime_Y(dst, str, off, len)) return false;
+    if (!ptime_char(' ', str, off, len)) return false;
+    if (!ptime_H(dst, str, off, len)) return false;
+    if (!ptime_char(':', str, off, len)) return false;
+    if (!ptime_M(dst, str, off, len)) return false;
+    if (!ptime_char(':', str, off, len)) return false;
+    if (!ptime_S(dst, str, off, len)) return false;
+    if (!ptime_char(' ', str, off, len)) return false;
+    if (!ptime_upto_end(str, off, len)) return false;
+    return true;
+}
+
+bool ptime_f28(struct exttm *dst, const char *str, off_t &off, ssize_t len) {
+    if (!ptime_d(dst, str, off, len)) return false;
+    if (!ptime_char(' ', str, off, len)) return false;
+    if (!ptime_b(dst, str, off, len)) return false;
+    if (!ptime_char(' ', str, off, len)) return false;
+    if (!ptime_Y(dst, str, off, len)) return false;
+    if (!ptime_char(' ', str, off, len)) return false;
+    if (!ptime_H(dst, str, off, len)) return false;
+    if (!ptime_char(':', str, off, len)) return false;
+    if (!ptime_M(dst, str, off, len)) return false;
+    if (!ptime_char(':', str, off, len)) return false;
+    if (!ptime_S(dst, str, off, len)) return false;
+    return true;
+}
+
+bool ptime_f29(struct exttm *dst, const char *str, off_t &off, ssize_t len) {
+    if (!ptime_d(dst, str, off, len)) return false;
+    if (!ptime_char(' ', str, off, len)) return false;
+    if (!ptime_b(dst, str, off, len)) return false;
+    if (!ptime_char(' ', str, off, len)) return false;
+    if (!ptime_Y(dst, str, off, len)) return false;
+    if (!ptime_char(' ', str, off, len)) return false;
+    if (!ptime_H(dst, str, off, len)) return false;
+    if (!ptime_char(':', str, off, len)) return false;
+    if (!ptime_M(dst, str, off, len)) return false;
+    if (!ptime_char(':', str, off, len)) return false;
+    if (!ptime_S(dst, str, off, len)) return false;
+    if (!ptime_char('.', str, off, len)) return false;
+    if (!ptime_L(dst, str, off, len)) return false;
+    return true;
+}
+
+bool ptime_f30(struct exttm *dst, const char *str, off_t &off, ssize_t len) {
+    if (!ptime_d(dst, str, off, len)) return false;
+    if (!ptime_char(' ', str, off, len)) return false;
+    if (!ptime_b(dst, str, off, len)) return false;
+    if (!ptime_char(' ', str, off, len)) return false;
+    if (!ptime_Y(dst, str, off, len)) return false;
+    if (!ptime_char(' ', str, off, len)) return false;
+    if (!ptime_H(dst, str, off, len)) return false;
+    if (!ptime_char(':', str, off, len)) return false;
+    if (!ptime_M(dst, str, off, len)) return false;
+    if (!ptime_char(':', str, off, len)) return false;
+    if (!ptime_S(dst, str, off, len)) return false;
+    if (!ptime_char(',', str, off, len)) return false;
+    if (!ptime_L(dst, str, off, len)) return false;
+    return true;
+}
+
+bool ptime_f31(struct exttm *dst, const char *str, off_t &off, ssize_t len) {
     if (!ptime_b(dst, str, off, len)) return false;
     if (!ptime_char(' ', str, off, len)) return false;
     if (!ptime_d(dst, str, off, len)) return false;
@@ -369,7 +503,7 @@ bool ptime_f23(struct exttm *dst, const char *str, off_t &off, ssize_t len) {
     return true;
 }
 
-bool ptime_f24(struct exttm *dst, const char *str, off_t &off, ssize_t len) {
+bool ptime_f32(struct exttm *dst, const char *str, off_t &off, ssize_t len) {
     if (!ptime_b(dst, str, off, len)) return false;
     if (!ptime_char(' ', str, off, len)) return false;
     if (!ptime_d(dst, str, off, len)) return false;
@@ -382,7 +516,7 @@ bool ptime_f24(struct exttm *dst, const char *str, off_t &off, ssize_t len) {
     return true;
 }
 
-bool ptime_f25(struct exttm *dst, const char *str, off_t &off, ssize_t len) {
+bool ptime_f33(struct exttm *dst, const char *str, off_t &off, ssize_t len) {
     if (!ptime_b(dst, str, off, len)) return false;
     if (!ptime_char(' ', str, off, len)) return false;
     if (!ptime_d(dst, str, off, len)) return false;
@@ -395,7 +529,7 @@ bool ptime_f25(struct exttm *dst, const char *str, off_t &off, ssize_t len) {
     return true;
 }
 
-bool ptime_f26(struct exttm *dst, const char *str, off_t &off, ssize_t len) {
+bool ptime_f34(struct exttm *dst, const char *str, off_t &off, ssize_t len) {
     if (!ptime_b(dst, str, off, len)) return false;
     if (!ptime_char(' ', str, off, len)) return false;
     if (!ptime_e(dst, str, off, len)) return false;
@@ -413,7 +547,7 @@ bool ptime_f26(struct exttm *dst, const char *str, off_t &off, ssize_t len) {
     return true;
 }
 
-bool ptime_f27(struct exttm *dst, const char *str, off_t &off, ssize_t len) {
+bool ptime_f35(struct exttm *dst, const char *str, off_t &off, ssize_t len) {
     if (!ptime_m(dst, str, off, len)) return false;
     if (!ptime_char('/', str, off, len)) return false;
     if (!ptime_d(dst, str, off, len)) return false;
@@ -428,7 +562,7 @@ bool ptime_f27(struct exttm *dst, const char *str, off_t &off, ssize_t len) {
     return true;
 }
 
-bool ptime_f28(struct exttm *dst, const char *str, off_t &off, ssize_t len) {
+bool ptime_f36(struct exttm *dst, const char *str, off_t &off, ssize_t len) {
     if (!ptime_m(dst, str, off, len)) return false;
     if (!ptime_char('/', str, off, len)) return false;
     if (!ptime_d(dst, str, off, len)) return false;
@@ -449,7 +583,7 @@ bool ptime_f28(struct exttm *dst, const char *str, off_t &off, ssize_t len) {
     return true;
 }
 
-bool ptime_f29(struct exttm *dst, const char *str, off_t &off, ssize_t len) {
+bool ptime_f37(struct exttm *dst, const char *str, off_t &off, ssize_t len) {
     if (!ptime_m(dst, str, off, len)) return false;
     if (!ptime_char('/', str, off, len)) return false;
     if (!ptime_e(dst, str, off, len)) return false;
@@ -466,7 +600,22 @@ bool ptime_f29(struct exttm *dst, const char *str, off_t &off, ssize_t len) {
     return true;
 }
 
-bool ptime_f30(struct exttm *dst, const char *str, off_t &off, ssize_t len) {
+bool ptime_f38(struct exttm *dst, const char *str, off_t &off, ssize_t len) {
+    if (!ptime_d(dst, str, off, len)) return false;
+    if (!ptime_char('/', str, off, len)) return false;
+    if (!ptime_b(dst, str, off, len)) return false;
+    if (!ptime_char('/', str, off, len)) return false;
+    if (!ptime_y(dst, str, off, len)) return false;
+    if (!ptime_char(' ', str, off, len)) return false;
+    if (!ptime_H(dst, str, off, len)) return false;
+    if (!ptime_char(':', str, off, len)) return false;
+    if (!ptime_M(dst, str, off, len)) return false;
+    if (!ptime_char(':', str, off, len)) return false;
+    if (!ptime_S(dst, str, off, len)) return false;
+    return true;
+}
+
+bool ptime_f39(struct exttm *dst, const char *str, off_t &off, ssize_t len) {
     if (!ptime_m(dst, str, off, len)) return false;
     if (!ptime_d(dst, str, off, len)) return false;
     if (!ptime_char(' ', str, off, len)) return false;
@@ -478,7 +627,7 @@ bool ptime_f30(struct exttm *dst, const char *str, off_t &off, ssize_t len) {
     return true;
 }
 
-bool ptime_f31(struct exttm *dst, const char *str, off_t &off, ssize_t len) {
+bool ptime_f40(struct exttm *dst, const char *str, off_t &off, ssize_t len) {
     if (!ptime_H(dst, str, off, len)) return false;
     if (!ptime_char(':', str, off, len)) return false;
     if (!ptime_M(dst, str, off, len)) return false;
@@ -487,14 +636,14 @@ bool ptime_f31(struct exttm *dst, const char *str, off_t &off, ssize_t len) {
     return true;
 }
 
-bool ptime_f32(struct exttm *dst, const char *str, off_t &off, ssize_t len) {
+bool ptime_f41(struct exttm *dst, const char *str, off_t &off, ssize_t len) {
     if (!ptime_M(dst, str, off, len)) return false;
     if (!ptime_char(':', str, off, len)) return false;
     if (!ptime_S(dst, str, off, len)) return false;
     return true;
 }
 
-bool ptime_f33(struct exttm *dst, const char *str, off_t &off, ssize_t len) {
+bool ptime_f42(struct exttm *dst, const char *str, off_t &off, ssize_t len) {
     if (!ptime_m(dst, str, off, len)) return false;
     if (!ptime_char('/', str, off, len)) return false;
     if (!ptime_d(dst, str, off, len)) return false;
@@ -507,7 +656,7 @@ bool ptime_f33(struct exttm *dst, const char *str, off_t &off, ssize_t len) {
     return true;
 }
 
-bool ptime_f34(struct exttm *dst, const char *str, off_t &off, ssize_t len) {
+bool ptime_f43(struct exttm *dst, const char *str, off_t &off, ssize_t len) {
     if (!ptime_Y(dst, str, off, len)) return false;
     if (!ptime_char('-', str, off, len)) return false;
     if (!ptime_m(dst, str, off, len)) return false;
@@ -516,14 +665,14 @@ bool ptime_f34(struct exttm *dst, const char *str, off_t &off, ssize_t len) {
     return true;
 }
 
-bool ptime_f35(struct exttm *dst, const char *str, off_t &off, ssize_t len) {
+bool ptime_f44(struct exttm *dst, const char *str, off_t &off, ssize_t len) {
     if (!ptime_Y(dst, str, off, len)) return false;
     if (!ptime_char('-', str, off, len)) return false;
     if (!ptime_m(dst, str, off, len)) return false;
     return true;
 }
 
-bool ptime_f36(struct exttm *dst, const char *str, off_t &off, ssize_t len) {
+bool ptime_f45(struct exttm *dst, const char *str, off_t &off, ssize_t len) {
     if (!ptime_Y(dst, str, off, len)) return false;
     if (!ptime_char('/', str, off, len)) return false;
     if (!ptime_m(dst, str, off, len)) return false;
@@ -532,13 +681,20 @@ bool ptime_f36(struct exttm *dst, const char *str, off_t &off, ssize_t len) {
     return true;
 }
 
-bool ptime_f37(struct exttm *dst, const char *str, off_t &off, ssize_t len) {
+bool ptime_f46(struct exttm *dst, const char *str, off_t &off, ssize_t len) {
     if (!ptime_Y(dst, str, off, len)) return false;
     if (!ptime_char('/', str, off, len)) return false;
     if (!ptime_m(dst, str, off, len)) return false;
     return true;
 }
 
+bool ptime_f47(struct exttm *dst, const char *str, off_t &off, ssize_t len) {
+    if (!ptime_s(dst, str, off, len)) return false;
+    if (!ptime_char('.', str, off, len)) return false;
+    if (!ptime_f(dst, str, off, len)) return false;
+    return true;
+}
+
 void ftime_f1(char *dst, off_t &off_inout, size_t len, const struct exttm &tm) {
     ftime_char(dst, off_inout, len, '@');
     ftime_at(dst, off_inout, len, tm);
@@ -767,6 +923,57 @@ void ftime_f15(char *dst, off_t &off_inout, size_t len, const struct exttm &tm)
 }
 
 void ftime_f16(char *dst, off_t &off_inout, size_t len, const struct exttm &tm) {
+    ftime_Y(dst, off_inout, len, tm);
+    ftime_char(dst, off_inout, len, ' ');
+    ftime_b(dst, off_inout, len, tm);
+    ftime_char(dst, off_inout, len, ' ');
+    ftime_d(dst, off_inout, len, tm);
+    ftime_char(dst, off_inout, len, ' ');
+    ftime_a(dst, off_inout, len, tm);
+    ftime_char(dst, off_inout, len, ' ');
+    ftime_H(dst, off_inout, len, tm);
+    ftime_char(dst, off_inout, len, ':');
+    ftime_M(dst, off_inout, len, tm);
+    ftime_char(dst, off_inout, len, ':');
+    ftime_S(dst, off_inout, len, tm);
+    ftime_char(dst, off_inout, len, '.');
+    ftime_L(dst, off_inout, len, tm);
+    dst[off_inout] = '\0';
+}
+
+void ftime_f17(char *dst, off_t &off_inout, size_t len, const struct exttm &tm) {
+    ftime_Y(dst, off_inout, len, tm);
+    ftime_char(dst, off_inout, len, ' ');
+    ftime_b(dst, off_inout, len, tm);
+    ftime_char(dst, off_inout, len, ' ');
+    ftime_d(dst, off_inout, len, tm);
+    ftime_char(dst, off_inout, len, ' ');
+    ftime_H(dst, off_inout, len, tm);
+    ftime_char(dst, off_inout, len, ':');
+    ftime_M(dst, off_inout, len, tm);
+    ftime_char(dst, off_inout, len, ':');
+    ftime_S(dst, off_inout, len, tm);
+    ftime_char(dst, off_inout, len, '.');
+    ftime_L(dst, off_inout, len, tm);
+    dst[off_inout] = '\0';
+}
+
+void ftime_f18(char *dst, off_t &off_inout, size_t len, const struct exttm &tm) {
+    ftime_Y(dst, off_inout, len, tm);
+    ftime_char(dst, off_inout, len, ' ');
+    ftime_b(dst, off_inout, len, tm);
+    ftime_char(dst, off_inout, len, ' ');
+    ftime_d(dst, off_inout, len, tm);
+    ftime_char(dst, off_inout, len, ' ');
+    ftime_H(dst, off_inout, len, tm);
+    ftime_char(dst, off_inout, len, ':');
+    ftime_M(dst, off_inout, len, tm);
+    ftime_char(dst, off_inout, len, ':');
+    ftime_S(dst, off_inout, len, tm);
+    dst[off_inout] = '\0';
+}
+
+void ftime_f19(char *dst, off_t &off_inout, size_t len, const struct exttm &tm) {
     ftime_a(dst, off_inout, len, tm);
     ftime_char(dst, off_inout, len, ' ');
     ftime_b(dst, off_inout, len, tm);
@@ -783,7 +990,7 @@ void ftime_f16(char *dst, off_t &off_inout, size_t len, const struct exttm &tm)
     dst[off_inout] = '\0';
 }
 
-void ftime_f17(char *dst, off_t &off_inout, size_t len, const struct exttm &tm) {
+void ftime_f20(char *dst, off_t &off_inout, size_t len, const struct exttm &tm) {
     ftime_a(dst, off_inout, len, tm);
     ftime_char(dst, off_inout, len, ' ');
     ftime_b(dst, off_inout, len, tm);
@@ -802,7 +1009,7 @@ void ftime_f17(char *dst, off_t &off_inout, size_t len, const struct exttm &tm)
     dst[off_inout] = '\0';
 }
 
-void ftime_f18(char *dst, off_t &off_inout, size_t len, const struct exttm &tm) {
+void ftime_f21(char *dst, off_t &off_inout, size_t len, const struct exttm &tm) {
     ftime_a(dst, off_inout, len, tm);
     ftime_char(dst, off_inout, len, ' ');
     ftime_b(dst, off_inout, len, tm);
@@ -821,7 +1028,7 @@ void ftime_f18(char *dst, off_t &off_inout, size_t len, const struct exttm &tm)
     dst[off_inout] = '\0';
 }
 
-void ftime_f19(char *dst, off_t &off_inout, size_t len, const struct exttm &tm) {
+void ftime_f22(char *dst, off_t &off_inout, size_t len, const struct exttm &tm) {
     ftime_a(dst, off_inout, len, tm);
     ftime_char(dst, off_inout, len, ' ');
     ftime_b(dst, off_inout, len, tm);
@@ -837,7 +1044,7 @@ void ftime_f19(char *dst, off_t &off_inout, size_t len, const struct exttm &tm)
     dst[off_inout] = '\0';
 }
 
-void ftime_f20(char *dst, off_t &off_inout, size_t len, const struct exttm &tm) {
+void ftime_f23(char *dst, off_t &off_inout, size_t len, const struct exttm &tm) {
     ftime_a(dst, off_inout, len, tm);
     ftime_char(dst, off_inout, len, ' ');
     ftime_b(dst, off_inout, len, tm);
@@ -855,7 +1062,7 @@ void ftime_f20(char *dst, off_t &off_inout, size_t len, const struct exttm &tm)
     dst[off_inout] = '\0';
 }
 
-void ftime_f21(char *dst, off_t &off_inout, size_t len, const struct exttm &tm) {
+void ftime_f24(char *dst, off_t &off_inout, size_t len, const struct exttm &tm) {
     ftime_d(dst, off_inout, len, tm);
     ftime_char(dst, off_inout, len, '/');
     ftime_b(dst, off_inout, len, tm);
@@ -876,7 +1083,7 @@ void ftime_f21(char *dst, off_t &off_inout, size_t len, const struct exttm &tm)
     dst[off_inout] = '\0';
 }
 
-void ftime_f22(char *dst, off_t &off_inout, size_t len, const struct exttm &tm) {
+void ftime_f25(char *dst, off_t &off_inout, size_t len, const struct exttm &tm) {
     ftime_d(dst, off_inout, len, tm);
     ftime_char(dst, off_inout, len, '/');
     ftime_b(dst, off_inout, len, tm);
@@ -893,7 +1100,90 @@ void ftime_f22(char *dst, off_t &off_inout, size_t len, const struct exttm &tm)
     dst[off_inout] = '\0';
 }
 
-void ftime_f23(char *dst, off_t &off_inout, size_t len, const struct exttm &tm) {
+void ftime_f26(char *dst, off_t &off_inout, size_t len, const struct exttm &tm) {
+    ftime_d(dst, off_inout, len, tm);
+    ftime_char(dst, off_inout, len, '-');
+    ftime_b(dst, off_inout, len, tm);
+    ftime_char(dst, off_inout, len, '-');
+    ftime_Y(dst, off_inout, len, tm);
+    ftime_char(dst, off_inout, len, ' ');
+    ftime_H(dst, off_inout, len, tm);
+    ftime_char(dst, off_inout, len, ':');
+    ftime_M(dst, off_inout, len, tm);
+    ftime_char(dst, off_inout, len, ':');
+    ftime_S(dst, off_inout, len, tm);
+    ftime_char(dst, off_inout, len, ' ');
+    ftime_z(dst, off_inout, len, tm);
+    dst[off_inout] = '\0';
+}
+
+void ftime_f27(char *dst, off_t &off_inout, size_t len, const struct exttm &tm) {
+    ftime_d(dst, off_inout, len, tm);
+    ftime_char(dst, off_inout, len, '-');
+    ftime_b(dst, off_inout, len, tm);
+    ftime_char(dst, off_inout, len, '-');
+    ftime_Y(dst, off_inout, len, tm);
+    ftime_char(dst, off_inout, len, ' ');
+    ftime_H(dst, off_inout, len, tm);
+    ftime_char(dst, off_inout, len, ':');
+    ftime_M(dst, off_inout, len, tm);
+    ftime_char(dst, off_inout, len, ':');
+    ftime_S(dst, off_inout, len, tm);
+    ftime_char(dst, off_inout, len, ' ');
+    ftime_Z(dst, off_inout, len, tm);
+    dst[off_inout] = '\0';
+}
+
+void ftime_f28(char *dst, off_t &off_inout, size_t len, const struct exttm &tm) {
+    ftime_d(dst, off_inout, len, tm);
+    ftime_char(dst, off_inout, len, ' ');
+    ftime_b(dst, off_inout, len, tm);
+    ftime_char(dst, off_inout, len, ' ');
+    ftime_Y(dst, off_inout, len, tm);
+    ftime_char(dst, off_inout, len, ' ');
+    ftime_H(dst, off_inout, len, tm);
+    ftime_char(dst, off_inout, len, ':');
+    ftime_M(dst, off_inout, len, tm);
+    ftime_char(dst, off_inout, len, ':');
+    ftime_S(dst, off_inout, len, tm);
+    dst[off_inout] = '\0';
+}
+
+void ftime_f29(char *dst, off_t &off_inout, size_t len, const struct exttm &tm) {
+    ftime_d(dst, off_inout, len, tm);
+    ftime_char(dst, off_inout, len, ' ');
+    ftime_b(dst, off_inout, len, tm);
+    ftime_char(dst, off_inout, len, ' ');
+    ftime_Y(dst, off_inout, len, tm);
+    ftime_char(dst, off_inout, len, ' ');
+    ftime_H(dst, off_inout, len, tm);
+    ftime_char(dst, off_inout, len, ':');
+    ftime_M(dst, off_inout, len, tm);
+    ftime_char(dst, off_inout, len, ':');
+    ftime_S(dst, off_inout, len, tm);
+    ftime_char(dst, off_inout, len, '.');
+    ftime_L(dst, off_inout, len, tm);
+    dst[off_inout] = '\0';
+}
+
+void ftime_f30(char *dst, off_t &off_inout, size_t len, const struct exttm &tm) {
+    ftime_d(dst, off_inout, len, tm);
+    ftime_char(dst, off_inout, len, ' ');
+    ftime_b(dst, off_inout, len, tm);
+    ftime_char(dst, off_inout, len, ' ');
+    ftime_Y(dst, off_inout, len, tm);
+    ftime_char(dst, off_inout, len, ' ');
+    ftime_H(dst, off_inout, len, tm);
+    ftime_char(dst, off_inout, len, ':');
+    ftime_M(dst, off_inout, len, tm);
+    ftime_char(dst, off_inout, len, ':');
+    ftime_S(dst, off_inout, len, tm);
+    ftime_char(dst, off_inout, len, ',');
+    ftime_L(dst, off_inout, len, tm);
+    dst[off_inout] = '\0';
+}
+
+void ftime_f31(char *dst, off_t &off_inout, size_t len, const struct exttm &tm) {
     ftime_b(dst, off_inout, len, tm);
     ftime_char(dst, off_inout, len, ' ');
     ftime_d(dst, off_inout, len, tm);
@@ -906,7 +1196,7 @@ void ftime_f23(char *dst, off_t &off_inout, size_t len, const struct exttm &tm)
     dst[off_inout] = '\0';
 }
 
-void ftime_f24(char *dst, off_t &off_inout, size_t len, const struct exttm &tm) {
+void ftime_f32(char *dst, off_t &off_inout, size_t len, const struct exttm &tm) {
     ftime_b(dst, off_inout, len, tm);
     ftime_char(dst, off_inout, len, ' ');
     ftime_d(dst, off_inout, len, tm);
@@ -919,7 +1209,7 @@ void ftime_f24(char *dst, off_t &off_inout, size_t len, const struct exttm &tm)
     dst[off_inout] = '\0';
 }
 
-void ftime_f25(char *dst, off_t &off_inout, size_t len, const struct exttm &tm) {
+void ftime_f33(char *dst, off_t &off_inout, size_t len, const struct exttm &tm) {
     ftime_b(dst, off_inout, len, tm);
     ftime_char(dst, off_inout, len, ' ');
     ftime_d(dst, off_inout, len, tm);
@@ -932,7 +1222,7 @@ void ftime_f25(char *dst, off_t &off_inout, size_t len, const struct exttm &tm)
     dst[off_inout] = '\0';
 }
 
-void ftime_f26(char *dst, off_t &off_inout, size_t len, const struct exttm &tm) {
+void ftime_f34(char *dst, off_t &off_inout, size_t len, const struct exttm &tm) {
     ftime_b(dst, off_inout, len, tm);
     ftime_char(dst, off_inout, len, ' ');
     ftime_e(dst, off_inout, len, tm);
@@ -950,7 +1240,7 @@ void ftime_f26(char *dst, off_t &off_inout, size_t len, const struct exttm &tm)
     dst[off_inout] = '\0';
 }
 
-void ftime_f27(char *dst, off_t &off_inout, size_t len, const struct exttm &tm) {
+void ftime_f35(char *dst, off_t &off_inout, size_t len, const struct exttm &tm) {
     ftime_m(dst, off_inout, len, tm);
     ftime_char(dst, off_inout, len, '/');
     ftime_d(dst, off_inout, len, tm);
@@ -965,7 +1255,7 @@ void ftime_f27(char *dst, off_t &off_inout, size_t len, const struct exttm &tm)
     dst[off_inout] = '\0';
 }
 
-void ftime_f28(char *dst, off_t &off_inout, size_t len, const struct exttm &tm) {
+void ftime_f36(char *dst, off_t &off_inout, size_t len, const struct exttm &tm) {
     ftime_m(dst, off_inout, len, tm);
     ftime_char(dst, off_inout, len, '/');
     ftime_d(dst, off_inout, len, tm);
@@ -986,7 +1276,7 @@ void ftime_f28(char *dst, off_t &off_inout, size_t len, const struct exttm &tm)
     dst[off_inout] = '\0';
 }
 
-void ftime_f29(char *dst, off_t &off_inout, size_t len, const struct exttm &tm) {
+void ftime_f37(char *dst, off_t &off_inout, size_t len, const struct exttm &tm) {
     ftime_m(dst, off_inout, len, tm);
     ftime_char(dst, off_inout, len, '/');
     ftime_e(dst, off_inout, len, tm);
@@ -1003,7 +1293,22 @@ void ftime_f29(char *dst, off_t &off_inout, size_t len, const struct exttm &tm)
     dst[off_inout] = '\0';
 }
 
-void ftime_f30(char *dst, off_t &off_inout, size_t len, const struct exttm &tm) {
+void ftime_f38(char *dst, off_t &off_inout, size_t len, const struct exttm &tm) {
+    ftime_d(dst, off_inout, len, tm);
+    ftime_char(dst, off_inout, len, '/');
+    ftime_b(dst, off_inout, len, tm);
+    ftime_char(dst, off_inout, len, '/');
+    ftime_y(dst, off_inout, len, tm);
+    ftime_char(dst, off_inout, len, ' ');
+    ftime_H(dst, off_inout, len, tm);
+    ftime_char(dst, off_inout, len, ':');
+    ftime_M(dst, off_inout, len, tm);
+    ftime_char(dst, off_inout, len, ':');
+    ftime_S(dst, off_inout, len, tm);
+    dst[off_inout] = '\0';
+}
+
+void ftime_f39(char *dst, off_t &off_inout, size_t len, const struct exttm &tm) {
     ftime_m(dst, off_inout, len, tm);
     ftime_d(dst, off_inout, len, tm);
     ftime_char(dst, off_inout, len, ' ');
@@ -1015,7 +1320,7 @@ void ftime_f30(char *dst, off_t &off_inout, size_t len, const struct exttm &tm)
     dst[off_inout] = '\0';
 }
 
-void ftime_f31(char *dst, off_t &off_inout, size_t len, const struct exttm &tm) {
+void ftime_f40(char *dst, off_t &off_inout, size_t len, const struct exttm &tm) {
     ftime_H(dst, off_inout, len, tm);
     ftime_char(dst, off_inout, len, ':');
     ftime_M(dst, off_inout, len, tm);
@@ -1024,14 +1329,14 @@ void ftime_f31(char *dst, off_t &off_inout, size_t len, const struct exttm &tm)
     dst[off_inout] = '\0';
 }
 
-void ftime_f32(char *dst, off_t &off_inout, size_t len, const struct exttm &tm) {
+void ftime_f41(char *dst, off_t &off_inout, size_t len, const struct exttm &tm) {
     ftime_M(dst, off_inout, len, tm);
     ftime_char(dst, off_inout, len, ':');
     ftime_S(dst, off_inout, len, tm);
     dst[off_inout] = '\0';
 }
 
-void ftime_f33(char *dst, off_t &off_inout, size_t len, const struct exttm &tm) {
+void ftime_f42(char *dst, off_t &off_inout, size_t len, const struct exttm &tm) {
     ftime_m(dst, off_inout, len, tm);
     ftime_char(dst, off_inout, len, '/');
     ftime_d(dst, off_inout, len, tm);
@@ -1044,7 +1349,7 @@ void ftime_f33(char *dst, off_t &off_inout, size_t len, const struct exttm &tm)
     dst[off_inout] = '\0';
 }
 
-void ftime_f34(char *dst, off_t &off_inout, size_t len, const struct exttm &tm) {
+void ftime_f43(char *dst, off_t &off_inout, size_t len, const struct exttm &tm) {
     ftime_Y(dst, off_inout, len, tm);
     ftime_char(dst, off_inout, len, '-');
     ftime_m(dst, off_inout, len, tm);
@@ -1053,14 +1358,14 @@ void ftime_f34(char *dst, off_t &off_inout, size_t len, const struct exttm &tm)
     dst[off_inout] = '\0';
 }
 
-void ftime_f35(char *dst, off_t &off_inout, size_t len, const struct exttm &tm) {
+void ftime_f44(char *dst, off_t &off_inout, size_t len, const struct exttm &tm) {
     ftime_Y(dst, off_inout, len, tm);
     ftime_char(dst, off_inout, len, '-');
     ftime_m(dst, off_inout, len, tm);
     dst[off_inout] = '\0';
 }
 
-void ftime_f36(char *dst, off_t &off_inout, size_t len, const struct exttm &tm) {
+void ftime_f45(char *dst, off_t &off_inout, size_t len, const struct exttm &tm) {
     ftime_Y(dst, off_inout, len, tm);
     ftime_char(dst, off_inout, len, '/');
     ftime_m(dst, off_inout, len, tm);
@@ -1069,13 +1374,20 @@ void ftime_f36(char *dst, off_t &off_inout, size_t len, const struct exttm &tm)
     dst[off_inout] = '\0';
 }
 
-void ftime_f37(char *dst, off_t &off_inout, size_t len, const struct exttm &tm) {
+void ftime_f46(char *dst, off_t &off_inout, size_t len, const struct exttm &tm) {
     ftime_Y(dst, off_inout, len, tm);
     ftime_char(dst, off_inout, len, '/');
     ftime_m(dst, off_inout, len, tm);
     dst[off_inout] = '\0';
 }
 
+void ftime_f47(char *dst, off_t &off_inout, size_t len, const struct exttm &tm) {
+    ftime_s(dst, off_inout, len, tm);
+    ftime_char(dst, off_inout, len, '.');
+    ftime_f(dst, off_inout, len, tm);
+    dst[off_inout] = '\0';
+}
+
 struct ptime_fmt PTIMEC_FORMATS[] = {
     { "@%@", ptime_f1, ftime_f1 },
     { "%Y-%m-%d %H:%M:%S", ptime_f2, ftime_f2 },
@@ -1092,28 +1404,38 @@ struct ptime_fmt PTIMEC_FORMATS[] = {
     { "%Y/%m/%d %H:%M:%S %z", ptime_f13, ftime_f13 },
     { "%Y/%m/%d %H:%M:%S%z", ptime_f14, ftime_f14 },
     { "%Y/%m/%d %H:%M", ptime_f15, ftime_f15 },
-    { "%a %b %d %H:%M:%S %Y", ptime_f16, ftime_f16 },
-    { "%a %b %d %H:%M:%S.%f %Y", ptime_f17, ftime_f17 },
-    { "%a %b %d %H:%M:%S %Z %Y", ptime_f18, ftime_f18 },
-    { "%a %b %d %H:%M:%S ", ptime_f19, ftime_f19 },
-    { "%a %b %d %H:%M:%S.%L ", ptime_f20, ftime_f20 },
-    { "%d/%b/%Y:%H:%M:%S +0000", ptime_f21, ftime_f21 },
-    { "%d/%b/%Y:%H:%M:%S %z", ptime_f22, ftime_f22 },
-    { "%b %d %H:%M:%S", ptime_f23, ftime_f23 },
-    { "%b %d %k:%M:%S", ptime_f24, ftime_f24 },
-    { "%b %d %l:%M:%S", ptime_f25, ftime_f25 },
-    { "%b %e, %Y %l:%M:%S %p", ptime_f26, ftime_f26 },
-    { "%m/%d/%y %H:%M:%S", ptime_f27, ftime_f27 },
-    { "%m/%d/%Y %I:%M:%S:%L %p %Z", ptime_f28, ftime_f28 },
-    { "%m/%e/%Y %l:%M:%S %p", ptime_f29, ftime_f29 },
-    { "%m%d %H:%M:%S", ptime_f30, ftime_f30 },
-    { "%H:%M:%S", ptime_f31, ftime_f31 },
-    { "%M:%S", ptime_f32, ftime_f32 },
-    { "%m/%d %H:%M:%S", ptime_f33, ftime_f33 },
-    { "%Y-%m-%d", ptime_f34, ftime_f34 },
-    { "%Y-%m", ptime_f35, ftime_f35 },
-    { "%Y/%m/%d", ptime_f36, ftime_f36 },
-    { "%Y/%m", ptime_f37, ftime_f37 },
+    { "%Y %b %d %a %H:%M:%S.%L", ptime_f16, ftime_f16 },
+    { "%Y %b %d %H:%M:%S.%L", ptime_f17, ftime_f17 },
+    { "%Y %b %d %H:%M:%S", ptime_f18, ftime_f18 },
+    { "%a %b %d %H:%M:%S %Y", ptime_f19, ftime_f19 },
+    { "%a %b %d %H:%M:%S.%f %Y", ptime_f20, ftime_f20 },
+    { "%a %b %d %H:%M:%S %Z %Y", ptime_f21, ftime_f21 },
+    { "%a %b %d %H:%M:%S ", ptime_f22, ftime_f22 },
+    { "%a %b %d %H:%M:%S.%L ", ptime_f23, ftime_f23 },
+    { "%d/%b/%Y:%H:%M:%S +0000", ptime_f24, ftime_f24 },
+    { "%d/%b/%Y:%H:%M:%S %z", ptime_f25, ftime_f25 },
+    { "%d-%b-%Y %H:%M:%S %z", ptime_f26, ftime_f26 },
+    { "%d-%b-%Y %H:%M:%S %Z", ptime_f27, ftime_f27 },
+    { "%d %b %Y %H:%M:%S", ptime_f28, ftime_f28 },
+    { "%d %b %Y %H:%M:%S.%L", ptime_f29, ftime_f29 },
+    { "%d %b %Y %H:%M:%S,%L", ptime_f30, ftime_f30 },
+    { "%b %d %H:%M:%S", ptime_f31, ftime_f31 },
+    { "%b %d %k:%M:%S", ptime_f32, ftime_f32 },
+    { "%b %d %l:%M:%S", ptime_f33, ftime_f33 },
+    { "%b %e, %Y %l:%M:%S %p", ptime_f34, ftime_f34 },
+    { "%m/%d/%y %H:%M:%S", ptime_f35, ftime_f35 },
+    { "%m/%d/%Y %I:%M:%S:%L %p %Z", ptime_f36, ftime_f36 },
+    { "%m/%e/%Y %l:%M:%S %p", ptime_f37, ftime_f37 },
+    { "%d/%b/%y %H:%M:%S", ptime_f38, ftime_f38 },
+    { "%m%d %H:%M:%S", ptime_f39, ftime_f39 },
+    { "%H:%M:%S", ptime_f40, ftime_f40 },
+    { "%M:%S", ptime_f41, ftime_f41 },
+    { "%m/%d %H:%M:%S", ptime_f42, ftime_f42 },
+    { "%Y-%m-%d", ptime_f43, ftime_f43 },
+    { "%Y-%m", ptime_f44, ftime_f44 },
+    { "%Y/%m/%d", ptime_f45, ftime_f45 },
+    { "%Y/%m", ptime_f46, ftime_f46 },
+    { "%s.%f", ptime_f47, ftime_f47 },
 
     { NULL, NULL }
 };
@@ -1133,6 +1455,9 @@ const char *PTIMEC_FORMAT_STR[] = {
     "%Y/%m/%d %H:%M:%S %z",
     "%Y/%m/%d %H:%M:%S%z",
     "%Y/%m/%d %H:%M",
+    "%Y %b %d %a %H:%M:%S.%L",
+    "%Y %b %d %H:%M:%S.%L",
+    "%Y %b %d %H:%M:%S",
     "%a %b %d %H:%M:%S %Y",
     "%a %b %d %H:%M:%S.%f %Y",
     "%a %b %d %H:%M:%S %Z %Y",
@@ -1140,6 +1465,11 @@ const char *PTIMEC_FORMAT_STR[] = {
     "%a %b %d %H:%M:%S.%L ",
     "%d/%b/%Y:%H:%M:%S +0000",
     "%d/%b/%Y:%H:%M:%S %z",
+    "%d-%b-%Y %H:%M:%S %z",
+    "%d-%b-%Y %H:%M:%S %Z",
+    "%d %b %Y %H:%M:%S",
+    "%d %b %Y %H:%M:%S.%L",
+    "%d %b %Y %H:%M:%S,%L",
     "%b %d %H:%M:%S",
     "%b %d %k:%M:%S",
     "%b %d %l:%M:%S",
@@ -1147,6 +1477,7 @@ const char *PTIMEC_FORMAT_STR[] = {
     "%m/%d/%y %H:%M:%S",
     "%m/%d/%Y %I:%M:%S:%L %p %Z",
     "%m/%e/%Y %l:%M:%S %p",
+    "%d/%b/%y %H:%M:%S",
     "%m%d %H:%M:%S",
     "%H:%M:%S",
     "%M:%S",
@@ -1155,6 +1486,7 @@ const char *PTIMEC_FORMAT_STR[] = {
     "%Y-%m",
     "%Y/%m/%d",
     "%Y/%m",
+    "%s.%f",
 
     NULL
 };
