@@ -32,17 +32,132 @@
 #include "config.h"
 
 #include <string>
-#include <algorithm>
 
 #include "auto_mem.hh"
 #include "lnav_log.hh"
 #include "view_curses.hh"
 #include "ansi_scrubber.hh"
 #include "lnav_config.hh"
-#include "attr_line.hh"
+#include "yajlpp.hh"
+#include "xterm-palette.hh"
 
 using namespace std;
 
+struct xterm_color {
+    short xc_id;
+    string xc_name;
+    rgb_color xc_color;
+    lab_color xc_lab_color;
+};
+
+static struct json_path_handler xterm_color_handler[] = {
+    json_path_handler("colorId")
+        .for_field(&nullobj<xterm_color>()->xc_id),
+    json_path_handler("name")
+        .for_field(&nullobj<xterm_color>()->xc_name),
+    json_path_handler("rgb/r")
+        .for_field(&nullobj<xterm_color>()->xc_color.rc_r),
+    json_path_handler("rgb/g")
+        .for_field(&nullobj<xterm_color>()->xc_color.rc_g),
+    json_path_handler("rgb/b")
+        .for_field(&nullobj<xterm_color>()->xc_color.rc_b),
+
+    json_path_handler()
+};
+
+static struct json_path_handler root_color_handler[] = {
+    json_path_handler("#/")
+        .with_obj_provider<xterm_color, vector<xterm_color>>(
+            [](const yajlpp_provider_context &ypc, vector<xterm_color> *palette) {
+                palette->resize(ypc.ypc_index + 1);
+                return &((*palette)[ypc.ypc_index]);
+            })
+        .with_children(xterm_color_handler),
+
+    json_path_handler()
+};
+
+static struct _xterm_colors {
+    _xterm_colors() {
+        yajlpp_parse_context ypc_xterm("xterm-palette.json", root_color_handler);
+        yajl_handle handle;
+
+        handle = yajl_alloc(&ypc_xterm.ypc_callbacks, NULL, &ypc_xterm);
+        ypc_xterm
+            .with_ignore_unused(true)
+            .with_obj(this->xc_palette)
+            .with_handle(handle);
+        ypc_xterm.parse((const unsigned char *) xterm_palette_json,
+                        strlen(xterm_palette_json));
+        ypc_xterm.complete_parse();
+        yajl_free(handle);
+
+        for (auto &xc : this->xc_palette) {
+            xc.xc_lab_color = lab_color(xc.xc_color);
+        }
+    };
+
+    short match_color(const lab_color &to_match) {
+        double lowest = 1000.0;
+        short lowest_id = -1;
+
+        for (auto &xc : this->xc_palette) {
+            double xc_delta = xc.xc_lab_color.deltaE(to_match);
+
+            if (lowest_id == -1) {
+                lowest = xc_delta;
+                lowest_id = xc.xc_id;
+                continue;
+            }
+
+            if (xc_delta < lowest) {
+                lowest = xc_delta;
+                lowest_id = xc.xc_id;
+            }
+        }
+
+        return lowest_id;
+    };
+
+    vector<xterm_color> xc_palette;
+} xterm_colors;
+
+bool rgb_color::from_str(const string_fragment &color,
+                         rgb_color &rgb_out,
+                         std::string &errmsg)
+{
+    if (color[0] == '#') {
+        switch (color.length()) {
+            case 4:
+                if (sscanf(color.data(), "#%1hx%1hx%1hx",
+                           &rgb_out.rc_r, &rgb_out.rc_g, &rgb_out.rc_b) == 3) {
+                    rgb_out.rc_r |= rgb_out.rc_r << 4;
+                    rgb_out.rc_g |= rgb_out.rc_g << 4;
+                    rgb_out.rc_b |= rgb_out.rc_b << 4;
+                    return true;
+                }
+                break;
+            case 7:
+                if (sscanf(color.data(), "#%2hx%2hx%2hx",
+                           &rgb_out.rc_r, &rgb_out.rc_g, &rgb_out.rc_b) == 3) {
+                    return true;
+                }
+                break;
+        }
+        errmsg = "Could not parse color: " + color.to_string();
+        return false;
+    }
+
+    for (const auto &xc : xterm_colors.xc_palette) {
+        if (color == xc.xc_name) {
+            rgb_out = xc.xc_color;
+            return true;
+        }
+    }
+
+    return false;
+}
+
 string_attr_type view_curses::VC_STYLE("style");
 string_attr_type view_curses::VC_GRAPHIC("graphic");
 
@@ -100,6 +215,14 @@ attr_line_t &attr_line_t::with_ansi_string(const char *str, ...)
     return *this;
 }
 
+attr_line_t &attr_line_t::with_ansi_string(const std::string &str)
+{
+    this->al_string = str;
+    scrub_ansi_string(this->al_string, this->al_attrs);
+
+    return *this;
+}
+
 attr_line_t &attr_line_t::append(const attr_line_t &al, text_wrap_settings *tws)
 {
     size_t start_len = this->al_string.length();
@@ -232,6 +355,21 @@ void attr_line_t::split_lines(std::vector<attr_line_t> &lines) const
     lines.emplace_back(this->subline(pos));
 }
 
+struct tab_mapping {
+    size_t tm_origin;
+    size_t tm_dst_start;
+    size_t tm_dst_end;
+
+    tab_mapping(size_t origin, size_t dst_start, size_t dst_end)
+        : tm_origin(origin), tm_dst_start(dst_start), tm_dst_end(dst_end) {
+
+    };
+
+    size_t length() const {
+        return this->tm_dst_end - this->tm_dst_start;
+    };
+};
+
 void view_curses::mvwattrline(WINDOW *window,
                               int y,
                               int x,
@@ -242,8 +380,8 @@ void view_curses::mvwattrline(WINDOW *window,
     int text_attrs, attrs, line_width;
     string_attrs_t &         sa   = al.get_attrs();
     string &                 line = al.get_string();
-    string_attrs_t::iterator iter;
-    std::map<size_t, size_t, std::greater<size_t> > tab_list;
+    string_attrs_t::const_iterator iter;
+    vector<tab_mapping> tab_list;
     int    tab_count = 0;
     char  *expanded_line;
     size_t exp_index = 0;
@@ -256,13 +394,15 @@ void view_curses::mvwattrline(WINDOW *window,
     expanded_line = (char *)alloca(line.size() + tab_count * 8 + 1);
 
     for (size_t lpc = 0; lpc < line.size(); lpc++) {
+        int exp_start_index = exp_index;
+
         switch (line[lpc]) {
         case '\t':
             do {
                 expanded_line[exp_index] = ' ';
                 exp_index += 1;
             } while (exp_index % 8);
-            tab_list[lpc] = exp_index;
+            tab_list.emplace_back(lpc, exp_start_index, exp_index);
             break;
 
         case '\r':
@@ -302,24 +442,24 @@ void view_curses::mvwattrline(WINDOW *window,
     stable_sort(sa.begin(), sa.end());
     for (iter = sa.begin(); iter != sa.end(); ++iter) {
         struct line_range attr_range = iter->sa_range;
-        std::map<size_t, size_t>::iterator tab_iter;
 
         require(attr_range.lr_start >= 0);
         require(attr_range.lr_end >= -1);
 
-        tab_iter = tab_list.lower_bound(attr_range.lr_start);
-        if (tab_iter != tab_list.end()) {
-            if ((size_t)attr_range.lr_start > tab_iter->first) {
-                attr_range.lr_start += (tab_iter->second - tab_iter->first) - 1;
+        for (auto tab_iter = tab_list.rbegin();
+             tab_iter != tab_list.rend();
+             ++tab_iter) {
+            if (tab_iter->tm_origin < attr_range.lr_start) {
+                attr_range.lr_start += tab_iter->length() - 1;
             }
         }
 
         if (attr_range.lr_end != -1) {
-            tab_iter = tab_list.lower_bound(attr_range.lr_end);
-            if (tab_iter != tab_list.end()) {
-                if ((size_t)attr_range.lr_end > tab_iter->first) {
-                    attr_range.lr_end += (
-                        tab_iter->second - tab_iter->first) - 1;
+            for (auto tab_iter = tab_list.rbegin();
+                 tab_iter != tab_list.rend();
+                 ++tab_iter) {
+                if (tab_iter->tm_origin < attr_range.lr_end) {
+                    attr_range.lr_end += tab_iter->length() - 1;
                 }
             }
         }
@@ -333,7 +473,7 @@ void view_curses::mvwattrline(WINDOW *window,
             attr_range.lr_end - lr.lr_start);
 
         if (attr_range.lr_end > attr_range.lr_start) {
-            string_attrs_t::iterator range_iter;
+            string_attrs_t::const_iterator range_iter;
             int awidth = attr_range.length();
             int color_pair = -1;
 
@@ -352,18 +492,22 @@ void view_curses::mvwattrline(WINDOW *window,
             if (attrs != 0) {
                 int x_pos = x + attr_range.lr_start;
                 int ch_width = min(awidth, (line_width - attr_range.lr_start));
-                chtype row_ch[ch_width + 1];
+                cchar_t row_ch[ch_width + 1];
 
-                mvwinchnstr(window, y, x_pos, row_ch, ch_width);
+                mvwin_wchnstr(window, y, x_pos, row_ch, ch_width);
                 for (int lpc = 0; lpc < ch_width; lpc++) {
                     if (color_pair > 0) {
-                        row_ch[lpc] &= ~A_COLOR;
-                        row_ch[lpc] |= (attrs & ~A_COLOR) | COLOR_PAIR(color_pair);
+                        row_ch[lpc].attr = attrs & ~A_COLOR;
+#ifdef NCURSES_EXT_COLORS
+                        row_ch[lpc].ext_color = color_pair;
+#else
+                        row_ch[lpc].attr |= COLOR_PAIR(color_pair);
+#endif
                     } else {
-                        row_ch[lpc] |= (attrs);
+                        row_ch[lpc].attr = attrs;
                     }
                 }
-                mvwaddchnstr(window, y, x_pos, row_ch, ch_width);
+                mvwadd_wchnstr(window, y, x_pos, row_ch, ch_width);
             }
             for (range_iter = iter;
                  range_iter != sa.end() && range_iter->sa_range == iter->sa_range;
@@ -387,7 +531,7 @@ void view_curses::mvwattrline(WINDOW *window,
 
 class color_listener : public lnav_config_listener {
     void reload_config() {
-        view_colors::singleton().init_roles();
+        view_colors::singleton().init_roles(0);
     }
 };
 
@@ -411,7 +555,7 @@ view_colors &view_colors::singleton(void)
     return s_vc;
 }
 
-view_colors::view_colors()
+view_colors::view_colors() : vc_color_pair_end(0)
 {
 }
 
@@ -419,6 +563,8 @@ bool view_colors::initialized = false;
 
 void view_colors::init(void)
 {
+    int color_pair_base = VC_ANSI_END;
+
     if (has_colors()) {
         static int ansi_colors_to_curses[] = {
             COLOR_BLACK,
@@ -433,7 +579,9 @@ void view_colors::init(void)
 
         start_color();
 
-        /* use_default_colors(); */
+        if (lnav_config.lc_ui_default_colors) {
+            use_default_colors();
+        }
         for (int fg = 0; fg < 8; fg++) {
             for (int bg = 0; bg < 8; bg++) {
                 if (fg == 0 && bg == 0)
@@ -445,14 +593,14 @@ void view_colors::init(void)
         }
 
         if (COLORS == 256) {
-            int color_pair_base = VC_ANSI_END;
+            int bg = (lnav_config.lc_ui_default_colors ? -1 : COLOR_BLACK);
 
             for (int z = 0; z < 6; z++) {
                 for (int x = 1; x < 6; x += 2) {
                     for (int y = 1; y < 6; y += 2) {
                         int fg = 16 + x + (y * 6) + (z * 6 * 6);
 
-                        init_pair(color_pair_base, fg, COLOR_BLACK);
+                        init_pair(color_pair_base, fg, bg);
                         color_pair_base += 1;
                     }
                 }
@@ -460,54 +608,87 @@ void view_colors::init(void)
         }
     }
 
-    singleton().init_roles();
+    singleton().init_roles(color_pair_base);
 
     initialized = true;
 }
 
-void view_colors::init_roles(void)
+inline int attr_for_colors(int &pair_base, short fg, short bg)
+{
+    int pair = ++pair_base;
+
+    if (lnav_config.lc_ui_default_colors) {
+        if (fg == COLOR_WHITE) {
+            fg = -1;
+        }
+        if (bg == COLOR_BLACK) {
+            bg = -1;
+        }
+    }
+
+    init_pair(pair, fg, bg);
+
+    return COLOR_PAIR(pair);
+}
+
+void view_colors::init_roles(int color_pair_base)
 {
     /* Setup the mappings from roles to actual colors. */
-    this->vc_role_colors[VCR_TEXT]   =
-        ansi_color_pair(COLOR_WHITE, COLOR_BLACK);
+    this->vc_role_colors[VCR_TEXT] =
+        attr_for_colors(color_pair_base, COLOR_WHITE, COLOR_BLACK);
     if (lnav_config.lc_ui_dim_text) {
         this->vc_role_colors[VCR_TEXT] |= A_DIM;
     }
     this->vc_role_colors[VCR_SEARCH] =
         this->vc_role_colors[VCR_TEXT] | A_REVERSE;
-    this->vc_role_colors[VCR_OK]      = ansi_color_pair(COLOR_GREEN, COLOR_BLACK) | A_BOLD;
-    this->vc_role_colors[VCR_ERROR]   = ansi_color_pair(COLOR_RED, COLOR_BLACK) | A_BOLD;
-    this->vc_role_colors[VCR_WARNING] = ansi_color_pair(COLOR_YELLOW, COLOR_BLACK) | A_BOLD;
-    this->vc_role_colors[VCR_ALT_ROW] = ansi_color_pair(COLOR_WHITE, COLOR_BLACK) | A_BOLD;
-    this->vc_role_colors[VCR_ADJUSTED_TIME] = ansi_color_pair(COLOR_MAGENTA, COLOR_BLACK);
-    this->vc_role_colors[VCR_SKEWED_TIME] = ansi_color_pair(COLOR_YELLOW, COLOR_BLACK) | A_UNDERLINE;
+    this->vc_role_colors[VCR_OK]      = attr_for_colors(color_pair_base, COLOR_GREEN, COLOR_BLACK) | A_BOLD;
+    this->vc_role_colors[VCR_ERROR]   = attr_for_colors(color_pair_base, COLOR_RED, COLOR_BLACK) | A_BOLD;
+    this->vc_role_colors[VCR_WARNING] = attr_for_colors(color_pair_base, COLOR_YELLOW, COLOR_BLACK) | A_BOLD;
+    this->vc_role_colors[VCR_ALT_ROW] = this->vc_role_colors[VCR_TEXT] | A_BOLD;
+    this->vc_role_colors[VCR_HIDDEN] = attr_for_colors(color_pair_base, COLOR_YELLOW, COLOR_BLACK);
+    this->vc_role_colors[VCR_ADJUSTED_TIME] = attr_for_colors(color_pair_base, COLOR_MAGENTA, COLOR_BLACK);
+    this->vc_role_colors[VCR_SKEWED_TIME] = attr_for_colors(color_pair_base, COLOR_YELLOW, COLOR_BLACK) | A_UNDERLINE;
+    this->vc_role_colors[VCR_OFFSET_TIME] = attr_for_colors(color_pair_base, COLOR_CYAN, COLOR_BLACK);
 
     this->vc_role_colors[VCR_STATUS] =
-        ansi_color_pair(COLOR_BLACK, COLOR_WHITE);
+        attr_for_colors(color_pair_base, COLOR_BLACK, COLOR_WHITE);
     this->vc_role_colors[VCR_WARN_STATUS] =
-        ansi_color_pair(COLOR_YELLOW, COLOR_WHITE) | A_BOLD;
+        attr_for_colors(color_pair_base, COLOR_YELLOW, COLOR_WHITE) | A_BOLD;
     this->vc_role_colors[VCR_ALERT_STATUS] =
-        ansi_color_pair(COLOR_RED, COLOR_WHITE) | A_BOLD;
+        attr_for_colors(color_pair_base, COLOR_RED, COLOR_WHITE) | A_BOLD;
     this->vc_role_colors[VCR_ACTIVE_STATUS] =
-        ansi_color_pair(COLOR_GREEN, COLOR_WHITE);
+        attr_for_colors(color_pair_base, COLOR_GREEN, COLOR_WHITE);
     this->vc_role_colors[VCR_ACTIVE_STATUS2] =
-        ansi_color_pair(COLOR_GREEN, COLOR_WHITE) | A_BOLD;
+        attr_for_colors(color_pair_base, COLOR_GREEN, COLOR_WHITE) | A_BOLD;
     this->vc_role_colors[VCR_BOLD_STATUS] =
-        ansi_color_pair(COLOR_BLACK, COLOR_WHITE) | A_BOLD;
+        attr_for_colors(color_pair_base, COLOR_BLACK, COLOR_WHITE) | A_BOLD;
     this->vc_role_colors[VCR_VIEW_STATUS] =
-        ansi_color_pair(COLOR_WHITE, COLOR_BLUE) | A_BOLD;
+        attr_for_colors(color_pair_base, COLOR_WHITE, COLOR_BLUE) | A_BOLD;
 
-    this->vc_role_colors[VCR_KEYWORD] = ansi_color_pair(COLOR_BLUE, COLOR_BLACK);
-    this->vc_role_colors[VCR_STRING] = ansi_color_pair(COLOR_GREEN, COLOR_BLACK) | A_BOLD;
-    this->vc_role_colors[VCR_COMMENT] = ansi_color_pair(COLOR_GREEN, COLOR_BLACK);
-    this->vc_role_colors[VCR_VARIABLE] = ansi_color_pair(COLOR_CYAN, COLOR_BLACK);
-    this->vc_role_colors[VCR_FILE] = ansi_color_pair(COLOR_BLUE, COLOR_BLACK);
+    this->vc_role_colors[VCR_KEYWORD] = attr_for_colors(color_pair_base, COLOR_BLUE, COLOR_BLACK);
+    this->vc_role_colors[VCR_STRING] = attr_for_colors(color_pair_base, COLOR_GREEN, COLOR_BLACK) | A_BOLD;
+    this->vc_role_colors[VCR_COMMENT] = attr_for_colors(color_pair_base, COLOR_GREEN, COLOR_BLACK);
+    this->vc_role_colors[VCR_VARIABLE] = attr_for_colors(color_pair_base, COLOR_CYAN, COLOR_BLACK);
+    this->vc_role_colors[VCR_SYMBOL] = attr_for_colors(color_pair_base, COLOR_MAGENTA, COLOR_BLACK);
+    this->vc_role_colors[VCR_RE_SPECIAL] = attr_for_colors(color_pair_base, COLOR_CYAN, COLOR_BLACK);
+    this->vc_role_colors[VCR_RE_REPEAT] = attr_for_colors(color_pair_base, COLOR_YELLOW, COLOR_BLACK);
+    this->vc_role_colors[VCR_FILE] = attr_for_colors(color_pair_base, COLOR_BLUE, COLOR_BLACK);
 
-    this->vc_role_colors[VCR_DIFF_DELETE]  = ansi_color_pair(COLOR_RED, COLOR_BLACK);
-    this->vc_role_colors[VCR_DIFF_ADD]     = ansi_color_pair(COLOR_GREEN, COLOR_BLACK);
-    this->vc_role_colors[VCR_DIFF_SECTION] = ansi_color_pair(COLOR_MAGENTA, COLOR_BLACK);
+    this->vc_role_colors[VCR_DIFF_DELETE]  = attr_for_colors(color_pair_base, COLOR_RED, COLOR_BLACK);
+    this->vc_role_colors[VCR_DIFF_ADD]     = attr_for_colors(color_pair_base, COLOR_GREEN, COLOR_BLACK);
+    this->vc_role_colors[VCR_DIFF_SECTION] = attr_for_colors(color_pair_base, COLOR_MAGENTA, COLOR_BLACK);
 
-    this->vc_role_colors[VCR_LOW_THRESHOLD] = ansi_color_pair(COLOR_BLACK, COLOR_GREEN);
-    this->vc_role_colors[VCR_MED_THRESHOLD] = ansi_color_pair(COLOR_BLACK, COLOR_YELLOW);
-    this->vc_role_colors[VCR_HIGH_THRESHOLD] = ansi_color_pair(COLOR_BLACK, COLOR_RED);
+    this->vc_role_colors[VCR_LOW_THRESHOLD] = attr_for_colors(color_pair_base, COLOR_BLACK, COLOR_GREEN);
+    this->vc_role_colors[VCR_MED_THRESHOLD] = attr_for_colors(color_pair_base, COLOR_BLACK, COLOR_YELLOW);
+    this->vc_role_colors[VCR_HIGH_THRESHOLD] = attr_for_colors(color_pair_base, COLOR_BLACK, COLOR_RED);
+
+    this->vc_color_pair_end = color_pair_base + 1;
+}
+
+int view_colors::ensure_color_pair(const rgb_color &rgb_fg, const rgb_color &rgb_bg)
+{
+    return attr_for_colors(
+        this->vc_color_pair_end,
+        xterm_colors.match_color(rgb_fg),
+        rgb_bg.empty() ? (short) COLOR_BLACK : xterm_colors.match_color(rgb_bg));
 }
