@@ -46,6 +46,14 @@ static T &resolve_root(yajlpp_parse_context *ypc)
 
     ptrdiff_t offset = (char *) jph->jph_simple_offset - (char *) NULL;
     char *retval = (char *) ypc->ypc_obj_stack.top();
+    char *ptr = retval + offset;
+
+    if (jph->jph_optional_wrapper) {
+        nonstd::optional<T> &wrapper = *(nonstd::optional<T> *) ptr;
+
+        wrapper = nonstd::make_optional(T());
+        return wrapper.value();
+    }
 
     return *((T *) (retval + offset));
 }
@@ -307,15 +315,21 @@ void yajlpp_validator_for_double(yajlpp_parse_context &ypc,
     }
 }
 
+template<typename T>
 int yajlpp_static_number(yajlpp_parse_context *ypc, long long num)
 {
-    long long &field_ptr = resolve_root<long long>(ypc);
+    T &field_ptr = resolve_root<T>(ypc);
 
     field_ptr = num;
 
     return 1;
 }
 
+template
+int yajlpp_static_number<long long>(yajlpp_parse_context *ypc, long long num);
+template
+int yajlpp_static_number<short>(yajlpp_parse_context *ypc, long long num);
+
 int yajlpp_static_decimal(yajlpp_parse_context *ypc, double num)
 {
     double &field_ptr = resolve_root<double>(ypc);
@@ -553,6 +567,8 @@ void yajlpp_parse_context::update_callbacks(const json_path_handler_base *orig_h
         return;
     }
 
+    this->ypc_sibling_handlers = orig_handlers;
+
     pcre_input pi(&this->ypc_path[0], 0, this->ypc_path.size() - 1);
 
     this->ypc_callbacks = DEFAULT_CALLBACKS;
@@ -731,10 +747,20 @@ int yajlpp_parse_context::handle_unused(void *ctx)
     }
 
     if (handler == NULL) {
+        const json_path_handler_base *accepted_handlers;
+
+        if (ypc->ypc_sibling_handlers) {
+            accepted_handlers = ypc->ypc_sibling_handlers;
+        } else {
+            accepted_handlers = ypc->ypc_handlers;
+        }
+
         fprintf(stderr, "  accepted paths --\n");
-        for (int lpc = 0; ypc->ypc_handlers[lpc].jph_path[0]; lpc++) {
-            fprintf(stderr, "    %s\n",
-                    ypc->ypc_handlers[lpc].jph_path);
+        for (int lpc = 0; accepted_handlers[lpc].jph_path[0]; lpc++) {
+            fprintf(stderr, "    %s %s -- %s\n",
+                    accepted_handlers[lpc].jph_path,
+                    accepted_handlers[lpc].jph_synopsis,
+                    accepted_handlers[lpc].jph_description);
         }
     }
 
