@@ -1,5 +1,12 @@
 #! /bin/bash
 
+run_test ${lnav_test} -n \
+    -c ":switch-to-view help" \
+    ${test_dir}/logfile_access_log.0
+
+check_output "switch-to-view help is not working" < ${top_srcdir}/test/expected_help.txt
+
+
 run_test ${lnav_test} -n \
     -c ":hide-fields foobar" \
     ${test_dir}/logfile_access_log.0
@@ -19,7 +26,7 @@ check_output "hide-fields with uri" <<EOF
 EOF
 
 run_test ${lnav_test} -n \
-    -c ":hide-fields c_ip cs_uri_stem" \
+    -c ":hide-fields access_log.c_ip access_log.cs_uri_stem" \
     ${test_dir}/logfile_access_log.0
 
 check_output "hide-fields with IP and uri" <<EOF
@@ -296,6 +303,22 @@ EOF
 cp ${test_dir}/logfile_multiline.0 logfile_append.0
 chmod ug+w logfile_append.0
 
+run_test ${lnav_test} -n -d /tmp/lnav-search.err \
+    -c "/goodbye" \
+    -c ";update generic_log set log_mark=1" \
+    -c ":filter-in Goodbye" \
+    -c ":append-to logfile_append.0" \
+    -c ":rebuild" \
+    -c ":next-mark search" \
+    logfile_append.0
+
+check_output "search after filter-in is not working" <<EOF
+2009-07-20 22:59:30,221:ERROR:Goodbye, World!
+EOF
+
+cp ${test_dir}/logfile_multiline.0 logfile_append.0
+chmod ug+w logfile_append.0
+
 run_test ${lnav_test} -n \
     -c ":filter-out Goodbye" \
     -c ":shexec echo '2009-07-20 22:59:30,221:ERROR:Goodbye, World!' >> logfile_append.0" \
@@ -361,13 +384,6 @@ error: filter limit reached, try combining filters with a pipe symbol (e.g. foo|
 EOF
 
 
-run_test ${lnav_test} -n \
-    -c ":switch-to-view help" \
-    ${test_dir}/logfile_access_log.0
-
-check_output "switch-to-view help is not working" < ${top_srcdir}/src/help.txt
-
-
 run_test ${lnav_test} -n \
     -c ":close" \
     ${test_dir}/logfile_access_log.0
@@ -721,6 +737,23 @@ check_output "histogram is not working?" <<EOF
 }
 EOF
 
+cp ${test_dir}/logfile_rollover.1 logfile_rollover.1.live
+chmod ug+w logfile_rollover.1.live
+touch -t 200711030923 logfile_rollover.1.live
+
+run_test ${lnav_test} -n \
+    -c ":shexec echo 'Jan  3 09:23:38 veridian automount[16442]: attempting to mount entry /auto/opt' >> logfile_rollover.1.live" \
+    -c ":rebuild" \
+    -c ":switch-to-view histogram" \
+    logfile_rollover.1.live
+
+check_output "rollover is not working with histogram" <<EOF
+ Thu Nov 03 09:20:00          1 normal         2 errors         0 warnings         0 marks
+ Thu Nov 03 09:45:00          1 normal         0 errors         0 warnings         0 marks
+ Fri Feb 03 09:20:00          0 normal         1 errors         0 warnings         0 marks
+ Wed Jan 03 09:20:00          1 normal         0 errors         0 warnings         0 marks
+EOF
+
 touch -t 200711030923 ${srcdir}/logfile_syslog.0
 run_test ${lnav_test} -n \
     -c ":switch-to-view histogram" \
